@@ -1,13 +1,15 @@
 from typing import Set, Type, Any, List, Dict, Optional, Iterable, TYPE_CHECKING
+import warnings as py_warnings
 
 from antlr4 import InputStream
 
 from . import messages
-from . import warnings
+from . import warnings # pylint: disable=reimported
 from .parser import sa_systemrdl
 from .core.ComponentVisitor import RootVisitor
 from .core.ExprVisitor import ExprVisitor
-from .core.properties import PropertyRuleBook, BuiltinUserProperty
+from .properties.rulebook import PropertyRuleBook
+from .properties.user_defined import ExternalUserProperty, LegacyExternalUserProperty
 from .core.namespace import NamespaceRegistry
 from .core.elaborate import ElabExpressionsListener, PrePlacementValidateListener, LateElabListener
 from .core.elaborate import StructuralPlacementListener, LateElabRevisitor
@@ -20,6 +22,7 @@ from . import preprocessor
 
 if TYPE_CHECKING:
     from .rdltypes.typing import RDLValue
+    from .udp import UDPDefinition
 
 
 class FileInfo:
@@ -108,54 +111,55 @@ class RDLCompiler:
         self.root = self.visitor.component # type: comp.Root # type: ignore
 
 
-    def define_udp(self, name, valid_type, valid_components=None, default=None):
-        # type: (str, List[Any], Optional[Set[Type[comp.Component]]], Any) -> None
+    def define_udp(
+            self, name: str, valid_type: Any,
+            valid_components: 'Optional[Set[Type[comp.Component]]]'=None,
+            default: Any=None
+        ) -> None:
+
+        py_warnings.warn(
+            "Use of RDLCompiler.define_udp() is deprecated. Use RDLCompiler.register_udp() instead.",
+            DeprecationWarning, stacklevel=2
+        )
+
+        if name in self.env.property_rules.rdl_properties:
+            raise ValueError("UDP definition's name '%s' conflicts with existing built-in RDL property")
+        if name in self.env.property_rules.user_properties:
+            raise ValueError("UDP '%s' has already been defined")
+
+        udp = LegacyExternalUserProperty(
+            self.env,
+            name,
+            valid_components,
+            valid_type,
+            default_assignment=default,
+            constr_componentwidth=False
+        )
+        self.env.property_rules.user_properties[udp.name] = udp
+
+    def register_udp(self, definition_cls: 'Type[UDPDefinition]', soft: bool=True) -> None:
         """
-        Pre-define a user-defined property.
-
-        This is the equivalent to the following RDL:
-
-        .. code-block:: none
-
-            property <name> {
-                type = <valid_type>;
-                component = <valid_components>;
-                default = <default>
-            };
+        Pre-register a User Defined Property into the compiler.
 
         Parameters
         ----------
-        name: str
-            Property name
-        valid_components: set
-            Set of :class:`~systemrdl.component.Component` types the UDP can be bound to.
-            If None, then UDP can be bound to all components.
-        valid_type: type
-            Assignment type that this UDP will enforce
-        default:
-            Default if a value is not specified when the UDP is bound to a component.
-            Value must be compatible with ``valid_type``
+        definition_cls: :class:`systemrdl.udp.UDPDefinition`
+            Reference to the container class that defines your new UDP.
+        soft: bool
+            Override to False to register the UDP as a hard definition.
 
+
+        .. versionadded:: 1.25
         """
-        if valid_components is None:
-            valid_components = {
-                comp.Field,
-                comp.Reg,
-                comp.Regfile,
-                comp.Addrmap,
-                comp.Mem,
-                comp.Signal,
-                #TODO constraint,
-            }
-
-        if name in self.env.property_rules.rdl_properties:
-            raise ValueError("name '%s' conflicts with existing built-in RDL property")
-
-        udp = BuiltinUserProperty(self.env, name, valid_components, (valid_type,), default)
+        if definition_cls.name in self.env.property_rules.rdl_properties:
+            raise ValueError("UDP definition's name '%s' conflicts with existing built-in RDL property")
+        if definition_cls.name in self.env.property_rules.user_properties:
+            raise ValueError("UDP '%s' has already been defined")
 
+        # Wrap definition with internal UDP object & register it
+        udp = ExternalUserProperty(self.env, definition_cls, soft)
         self.env.property_rules.user_properties[udp.name] = udp
 
-
     def list_udps(self) -> List[str]:
         """
         List all user-defined properties encountered by the compiler.
@@ -163,7 +167,12 @@ class RDLCompiler:
 
         .. versionadded:: 1.12
         """
-        return list(self.env.property_rules.user_properties.keys())
+        udps = []
+        for udp_name, udp in self.env.property_rules.user_properties.items():
+            if isinstance(udp, ExternalUserProperty) and udp.is_soft:
+                continue
+            udps.append(udp_name)
+        return udps
 
 
     def preprocess_file(self, path: str, incl_search_paths: Optional[List[str]]=None) -> FileInfo:
